/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2007 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <glib.h>
#include <sys/msg.h>
#include <errno.h>
#include <stdlib.h>
#include "Communication.h"
#include "Commun.h"
#include "../Debug.h"
#include "../Thread.h"
#include "../Intrus.h"
#include "../Notification_Port.h"
#include "Socket.h"
#include "Semaphore.h"
#include "Scan_Port.h"
#include "Scan_Signature.h"
#include "Sous_Reseau_Autorise.h"
#include "Initialisation_Com.h"
#include "Notification_Equipement.h"
#include "../Restoration.h"
#include "../Nessus.h"
#include "Declaration_Constante.h"
#include "../Scan_Equipement.h"


void Arret_Demande (gpointer * Message)
{
	printf("Arret_Demande\n");
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[S] Demande d'arret (AutoScan) Socket ouvert[%d]  Thread[%d]", Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan));
	printf ("Stopping %s (http://AutoScan.fr) %s  pid(%d)\n", VERSION_AGENT, version_compiled, getpid ());
	printf("ID socket : %d\n",client_id);
	shutdown (client_id, 0);
  	close (client_id);
	exit(EXIT_SUCCESS);
}


struct Struct_Modele_Communication Liste_Communication[] = {
	{AGENT_VERSION, NULL,"AGENT_VERSION"},
	{AJOUT_PORT, &Ajout_Un_Port,"AJOUT_PORT"},
	{AJOUT_SIGNATURE, NULL,"AJOUT_SIGNATURE"},
	{RESTAURATION_MACHINE, &Ajout_Equipement_Restoration,"RESTAURATION_MACHINE"},
	{AJOUT_RESEAU_PRIVEE, &Ajout_Sous_Reseau_Autorise,"AJOUT_RESEAU_PRIVEE"},
	{DEMANDE_LISTE_INTERFACE, &Envoie_Liste_Interface,"DEMANDE_LISTE_INTERFACE"},
	{NOUVELLE_INTERFACE, NULL,"NOUVELLE_INTERFACE"},
	{FIN_LISTE_INTERFACE, NULL,"FIN_LISTE_INTERFACE"},
	{DEMARRAGE_AGENT, &Demarrage_Agent,"DEMARRAGE_AGENT"},
	{DEMANDE_ARRET_AGENT, &Arret_Demande,"DEMANDE_ARRET_AGENT"},
	{RETOUR_MESSAGE, NULL,"RETOUR_MESSAGE"},
	{RETOUR_ERREUR_CRITIQUE, NULL,"RETOUR_ERREUR_CRITIQUE"},
	{INFO_AGENT, NULL,"INFO_AGENT"},
	{NOUVEAU_EQUIPEMENT, NULL,"NOUVEAU_EQUIPEMENT"},
	{DEMANDE_NESSUS, &Initialisation_Nessus,"DEMANDE_NESSUS"},
	{DEMARRAGE_NESSUS, NULL,"DEMARRAGE_NESSUS"},
	{RETOUR_NESSUS_INFO, NULL,"RETOUR_NESSUS_INFO"},
	{RETOUR_NESSUS_DONNEE, NULL,"RETOUR_NESSUS_DONNEE"},
	{RETOUR_NESSUS_FIN, NULL,"RETOUR_NESSUS_FIN"},
	{DEMANDE_RAFRAICHIR_ETAT_EQUIMEMENT, &Rafraichir_Etat_Equipement,"DEMANDE_RAFRAICHIR_ETAT_EQUIMEMENT"},
	{RETOUR_RAFRAICHIR_ETAT_EQUIMEMENT, NULL,"RETOUR_RAFRAICHIR_ETAT_EQUIMEMENT"},
	{DEMANDE_RAFRAICHIR_LISTE_PORTS, NULL,"DEMANDE_RAFRAICHIR_LISTE_PORTS"},
	{RETOUR_RAFRAICHIR_LISTE_PORTS, NULL,"RETOUR_RAFRAICHIR_LISTE_PORTS"},
	{DEMANDE_RAFRAICHIR_ENREGISTREMENT_SAMBA, NULL,"DEMANDE_RAFRAICHIR_ENREGISTREMENT_SAMBA"},
	{RETOUR_RAFRAICHIR_ENREGISTREMENT_SAMBA, NULL,"RETOUR_RAFRAICHIR_ENREGISTREMENT_SAMBA"},
	{DEMANDE_RAFRAICHIR_SIGNATURE, NULL,"DEMANDE_RAFRAICHIR_SIGNATURE"},
	{RETOUR_RAFRAICHIR_SIGNATURE, NULL,"RETOUR_RAFRAICHIR_SIGNATURE"},
	{DEMANDE_NOTIFICATION_WARNING_ETAT_EQUIPEMENT, NULL,"DEMANDE_NOTIFICATION_WARNING_ETAT_EQUIPEMENT"},
	{DEMANDE_NOTIFICATION_WARNING_ETAT_PORT, NULL,"DEMANDE_NOTIFICATION_WARNING_ETAT_PORT"},
	{DEMANDE_NOTIFICATION_CRITIQUE_ETAT_EQUIPEMENT, &Ajout_Notification_Equipement,"DEMANDE_NOTIFICATION_CRITIQUE_ETAT_EQUIPEMENT"},
	{DEMANDE_NOTIFICATION_CRITIQUE_ETAT_PORT, &Ajout_Notification_Port,"DEMANDE_NOTIFICATION_CRITIQUE_ETAT_PORT"},
	{RETOUR_NOTIFICATION_CHANGEMENT_ETAT_EQUIPEMENT, NULL,"RETOUR_NOTIFICATION_CHANGEMENT_ETAT_EQUIPEMENT"},
	{RETOUR_NOTIFICATION_CHANGEMENT_ETAT_PORT, NULL,"RETOUR_NOTIFICATION_CHANGEMENT_ETAT_PORT"},
	{ANNULATION_NOTIFICATION_ETAT_EQUIPEMENT, &Enlever_Notification_Equipement,"ANNULATION_NOTIFICATION_ETAT_EQUIPEMENT"},
	{ANNULATION_NOTIFICATION_ETAT_PORT, &Enlever_Notification_Port,"ANNULATION_NOTIFICATION_ETAT_PORT"},
	{ACTIVE_ALERTE_INTRUS, &Active_Alerte_Intrusion,"ACTIVE_ALERTE_INTRUS"},
	{DESACTIVE_ALERTE_INTRUS, &Desactive_Alerte_Intrusion,"DESACTIVE_ALERTE_INTRUS"},
	{ALERTE_INTRUS, NULL,"ALERTE_INTRUS"},
	{DEMANDE_MODIFIE_NIVEAU_DEBUG, NULL,"DEMANDE_MODIFIE_NIVEAU_DEBUG"},
	{RETOUR_LISTE_SNMP, NULL,"RETOUR_LISTE_SNMP"},
	{DEMANDE_LANCEMENT_SCRIPT, NULL,"DEMANDE_LANCEMENT_SCRIPT"},
	{RETOUR_SCRIPT, NULL,"RETOUR_SCRIPT"},
	{RETOUR_SCRIPT_TERMINER, NULL,"RETOUR_SCRIPT_TERMINER"},
	{DEMANDE_RAFRAICHIR_EQUIPEMENT, &Demande_Rafraichir_Equipement,"DEMANDE_RAFRAICHIR_EQUIPEMENT"},
	{RETOUR_RAFRAICHIR_EQUIPEMENT, NULL,"RETOUR_RAFRAICHIR_EQUIPEMENT"},
	{IP_RETROUVER, NULL,"IP_RETROUVER"},
	{IP_PERDU, NULL,"IP_PERDU"},
	{IP_MODIFIER, NULL,"IP_MODIFIER"},
};


void     *Communication_Reception_Donnee_Boucle (void)
{
	struct struct_Info_Entete Entete;
	gint      Taille_Donnee_Recue,Ckecksum;
	gchar     Donnee_Brut[TAILLE_MAX_DONNEE_BRUT];
	guint     Tentative,Taille_Donnee_Recue_Part;
	
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Communication_Reception_Donnee_Boucle    pid(%d)", Mon_Pid);
	while (1) {
	
		Taille_Donnee_Recue = recvfrom (client_id, &Entete, sizeof (struct struct_Info_Entete), 0, NULL, 0);
	
		if (Taille_Donnee_Recue>0){
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s]  Taille:[%d]", Liste_Communication[Entete.ID_Info].Debug, Entete.Taille_Donnee);
			if (Entete.Taille_Donnee > TAILLE_MAX_DONNEE_BRUT)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s]  Taille:[%d]", Liste_Communication[Entete.ID_Info].Debug, Entete.Taille_Donnee);
			Taille_Donnee_Recue = recvfrom (client_id, &Donnee_Brut, Entete.Taille_Donnee, 0, NULL, 0);
			if ((Taille_Donnee_Recue == -1)||(Taille_Donnee_Recue == 0))
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "[%s] Taille Recupere Entete : (%d)", strerror (errno),Taille_Donnee_Recue);
			
			
		
		
			if (Entete.Taille_Donnee!=Taille_Donnee_Recue){
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete.Taille_Donnee!=Taille_Donnee_Recue [%d!=%d]", Entete.Taille_Donnee,Taille_Donnee_Recue);
				
					Tentative = 0;
					do{
					
						Tentative++;
						sleep(1);
						Taille_Donnee_Recue_Part = recvfrom (client_id, &Donnee_Brut[Taille_Donnee_Recue], Entete.Taille_Donnee-Taille_Donnee_Recue, 0, NULL, 0);
						debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Taille_Donnee_Recue_Part recupere en plus [%d]", Taille_Donnee_Recue_Part);
						Taille_Donnee_Recue = Taille_Donnee_Recue + Taille_Donnee_Recue_Part;
					}while ((Entete.Taille_Donnee>Taille_Donnee_Recue)&&(Tentative!=10));
				
					if (Entete.Taille_Donnee!=Taille_Donnee_Recue)
						debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Entete.Taille_Donnee!=Taille_Donnee_Recue [%d!=%d]", Entete.Taille_Donnee,Taille_Donnee_Recue);
				}	
			
			Ckecksum = Calculer_Ckecksum(Donnee_Brut,Taille_Donnee_Recue);
			if (Ckecksum!=Entete.Ckecksum)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Ckecksum!=Entete.Ckecksum [%08X!=%08X]", Ckecksum, Entete.Ckecksum);

		
			if (Liste_Communication[Entete.ID_Info].Gestion_Communication != NULL)
				(*Liste_Communication[Entete.ID_Info].Gestion_Communication) ((gpointer *) &Donnee_Brut);
			else
				debug_printf (D_WARN_USER, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Reception Message sans action defini");
		}
	
	}
}


void Communication_Initialisation (void)
{
#define NB_CARACTERE_IDENTIFICATION 25
	gchar     Identification[NB_CARACTERE_IDENTIFICATION];
	gint      NbOctet;


	Mon_Pid = getpid ();

	Socket_Service = create_tcp_server (4555, 1);
	if (-1 == (client_id = accept (Socket_Service, NULL, 0)))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "accept", strerror (errno));

	shutdown (Socket_Service, SHUT_RDWR);
  	close (Socket_Service);
	NbOctet = send (client_id, VERSION_AGENT, sizeof (VERSION_AGENT), 0);
	if (NbOctet < 0)
		printf ("send  : %s\n", strerror (errno));

	NbOctet = recvfrom (client_id, &Identification, sizeof (gchar) * NB_CARACTERE_IDENTIFICATION, 0, NULL, 0);
	if (NbOctet == -1)
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "read", strerror (errno));


	
	
	/*if ((File_Info = msgget (KEY_FILE, 0)) == -1)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s \nRun 'AutoScan'", strerror (errno));
	*/
	
	demmarrage_Thread (Communication_Reception_Donnee_Boucle, NULL);
}
