/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2007 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <netinet/if_ether.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#include <linux/sockios.h>
#include <errno.h>
#include "Initialisation_Com.h"
#include "Commun.h"
#include "../Debug.h"
#include "../Declaration_Constante.h"
#include "../Notification_Equipement.h"
#include "../Notification_Port.h"
#include "Reseau.h"
#include "Thread_Detect.h"
#include "Thread.h"
#include "../Info.h"
#include "Semaphore.h"
#include "Scan_Signature.h"
#include "Scan_Service.h"

void Envoie_Version_Agent (void)
{
	struct struct_Agent_Version Message;
	gint      pid;

	pid = getpid ();
	ERROR_if (strlen (VERSION_AGENT) > TAILLE_MAX_VERSION_AGENT);
	strcpy (Message.Version, VERSION_AGENT);
	Message.pid = pid;
	Communication_Envoie_Donnee (AGENT_VERSION, (gpointer *) & Message, sizeof (Message));
}

void Demarrage_Agent (gpointer * Donnee)
{
	ERROR_if (Donnee == NULL);
	memcpy (&Info_Demarrage, Donnee, sizeof (struct struct_Demarrage_Agent));
	if (Info_Demarrage.Normal_Mode){
		Time_Out_Service = 25;
		Time_Out_Connect_Service = 25;
	}
	else {
		Time_Out_Connect_Service = 10;
		Time_Out_Service = 10;
	}
	Debug = Info_Demarrage.Niveau_Debugage;
	Get_My_Ip ();
	Get_My_MAC ();
	Initialisation_Semaphore_Gestion_Thread ();
	demmarrage_Thread (&Boucle_Scan, NULL);
	demmarrage_Thread (&Thread_Detect, NULL);
	demmarrage_Thread (&Thread_Notification_Equipement, NULL);
	demmarrage_Thread (&Thread_Notification_Port, NULL);
	demmarrage_Thread (&Info_Agent, NULL);	
	demmarrage_Thread (&Thread_Rafraichir_Tout_Equipement, NULL);

	demmarrage_Thread (&Ping_Detect, NULL);	
}

void Envoie_Liste_Interface (gpointer * Donnee)
{
	FILE     *Fichier_Interface = NULL;
	struct struct_Nouvelle_Interface Message;
	struct struct_Fin_Liste_Interface Message_Fin;
	struct ifreq ifr;
	struct sockaddr_in sin;
	struct in_addr in;
	gchar     buf[512], *Interface, *fin;
	int       Socket;


	ERROR_if (Donnee == NULL);
	if ((Fichier_Interface = fopen ("/proc/net/dev", "r")) == NULL)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "/proc/net/dev : %s", strerror (errno));
	fgets (buf, sizeof (buf), Fichier_Interface);
	fgets (buf, sizeof (buf), Fichier_Interface);
	while (fgets (buf, sizeof (buf), Fichier_Interface)) {
		Interface = buf;
		while (Interface[0] == ' ')
			Interface++;
		fin = index (buf, ':');
		fin[0] = 0;
		debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Interface : %s", Interface);
		if (strcmp (Interface, "lo") != 0) {
		
			Socket = socket_control (AF_INET, SOCK_STREAM, 0);
			if (Socket == -1) {
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
				continue;
			}
			strcpy (ifr.ifr_name, Interface);
			if (ioctl (Socket, SIOCGIFADDR, &ifr) != -1) {
				memcpy (&sin, &ifr.ifr_addr, sizeof (struct sockaddr_in));
				in.s_addr = sin.sin_addr.s_addr;
			
				strcpy (Message.Interface, Interface);
			
				strcpy (Message.Ip, inet_ntoa (in));
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Nouvelle Interface : interface[%s] ip[%s]", Message.Interface,Message.Ip);
				Communication_Envoie_Donnee (NOUVELLE_INTERFACE, (gpointer *) & Message, sizeof (Message));
			}
			close_control (Socket);
		}
	}
	Communication_Envoie_Donnee (FIN_LISTE_INTERFACE, (gpointer *) & Message_Fin, sizeof (Message_Fin));
}
