/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2007 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Scan_Port.h"
#include "Debug.h"
#include "Info.h"
#include "Com/Initialisation_Com.h"

char      Syn[] = { 0x45, 0x00, 0x00, 0x28, 0x00, 0x00, 0x00, 0x00, 0x7F, 0x06, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x50, 0x02, 0x1F, 0xFE, 0x00, 0x00, 0x00, 0x00 };


void Envoie_Trame_Scan_Port (struct Struct_Liste_Equipement *Equipement, gint Num_Port, gint Socket, struct sockaddr *sock)
{
	unsigned short Id_Ip, Port;
	unsigned int MonNumSeq;
	struct struct_trame PacketScan;
	unsigned short CheckSum_Calc;

	ERROR_if (Equipement == NULL);
	ERROR_if (sock == NULL);

	Id_Ip = (unsigned short) random ();
	MonNumSeq = (int) random ();
	Port = ((unsigned short) random ());
	Port = (Port & 0x0FFF) + 0xF000;
	MonNumSeq = htonl (MonNumSeq);
	memcpy (&PacketScan.Dest_MAC, &Equipement->Adresse_MAC, ETH_ALEN);
	memcpy (&PacketScan.Source_MAC, Mon_Adresse_MAC, ETH_ALEN);
	PacketScan.Proto = htons (TRAME_IP);
	memcpy (&PacketScan.Datagramme, Syn, 40);
	PacketScan.Datagramme[4] = (Id_Ip >> 8) & 0xFF;
	PacketScan.Datagramme[5] = Id_Ip & 0xFF;
	memcpy (&PacketScan.Datagramme[12], &Mon_Adresse_Ip, IP_ADDR_LEN);
	memcpy (&PacketScan.Datagramme[16], &Equipement->Adresse_Ip, IP_ADDR_LEN);
	CheckSum_Calc = htons (ckecksum (&PacketScan.Datagramme[0], 20, 0));
	memcpy (&PacketScan.Datagramme[10], &CheckSum_Calc, 2);
	PacketScan.Datagramme[20] = (Port >> 8) & 0xFF;
	PacketScan.Datagramme[21] = Port & 0xFF;
	PacketScan.Datagramme[22] = (Num_Port >> 8) & 0xFF;
	PacketScan.Datagramme[23] = Num_Port & 0xFF;
	memcpy (&PacketScan.Datagramme[24], &MonNumSeq, 4);
	CheckSum_Calc = htons (ckecksum (&PacketScan.Datagramme[12], 24, 26));
	memcpy (&PacketScan.Datagramme[36], &CheckSum_Calc, 2);
	Nouveau_Octet_Envoye (54);
	if ((sendto (Socket, &PacketScan, 54, 0, sock, sizeof (struct sockaddr))) < 0) {
		if ((errno != EPIPE) && (errno != ESPIPE))
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
	}
}


gint Nombre_De_Port_Scanner (struct Struct_Liste_Equipement *Equipement)
{
	ERROR_if (Equipement == NULL);
	return g_list_length (Equipement->GList_Port);
}


gint Nombre_De_Port_Avec_Etat (struct Struct_Liste_Equipement * Equipement, gint Etat)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      Nb_Objet_Liste, i, Nb_Port;

	ERROR_if (Equipement == NULL);
	Nb_Port = 0;
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Etat_Port == Etat)
			Nb_Port++;
	}
	return Nb_Port;
}


gint Cherche_Un_Port_Avec_Etat (struct Struct_Liste_Equipement * Equipement, gint Etat)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Etat_Port == Etat)
			return Port_Equipement->Num_Port;
	}
	return -1;
}


gboolean Cherche_Si_Port_Ouvert (struct Struct_Liste_Equipement * Equipement, gint Num_Port)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Num_Port == Num_Port)
			return (Port_Equipement->Etat_Port == OUVERT);
	}
	return FALSE;	
}


struct Struct_Port *Recupere_Pointeur_Port_Glist (struct Struct_Liste_Equipement *Equipement, gint Num_Port)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Num_Port == Num_Port)
			return Port_Equipement;
	}
	return NULL;
}


void Modifie_Etat_Port_Equipement (struct Struct_Liste_Equipement *Equipement, gint Num_Port, gint Nouvelle_Etat)
{
	struct Struct_Port *Port_Equipement = NULL;
	struct struct_Retour_Rachaichir_Liste_Ports Message;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	i = 0;
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	while (i < Nb_Objet_Liste) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Num_Port == Num_Port) {
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Modifier etat port : %d  [%d]", Port_Equipement->Num_Port, Nouvelle_Etat);
			Port_Equipement->Etat_Port = Nouvelle_Etat;
		
			if ((!Equipement->Scan_Port_En_Cours) && (Nouvelle_Etat == OUVERT)) {
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Compteur du port %d mis a 0", Port_Equipement->Num_Port);
				Port_Equipement->Compteur_Notification = 0;
				if (Port_Equipement->Alerte_Port_Active) {
					debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Rafraichir etat port : %d  [%d]", Port_Equipement->Num_Port, Nouvelle_Etat);
					bzero (&Message, sizeof (struct struct_Retour_Rachaichir_Liste_Ports));
					Message.ID_Reseau = Info_Demarrage.ID_Reseau;
					memcpy (Message.ID_Equipement, Equipement->ID_Equipement, MAC_ADDR_LEN + IP_ADDR_LEN + 1);
					Message.Num_Port = Num_Port;
					Message.Etat_Port = TRUE;
					time (&Port_Equipement->Derniere_Modification_Etat_Port);
					Message.Derniere_Modification_Etat_Port = Port_Equipement->Derniere_Modification_Etat_Port;
					Communication_Envoie_Donnee (RETOUR_RAFRAICHIR_LISTE_PORTS, (gpointer *) & Message, sizeof (Message));
					Port_Equipement->Alerte_Port_Active = FALSE;
				}
			}
		
			return;
		}
		i++;
	}
	debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Port %d Inconnu", Num_Port);

}


void Initialisation_Liste_Port_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Liste ports dans la liste : %d", Nb_Objet_Liste);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Met a 0 le port : %d", Port_Equipement->Num_Port);
		Port_Equipement->Etat_Port = FIREWALLED;
	}
}
gboolean Test_Retour_Port_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port_Equipement = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port_Equipement = g_list_nth_data (Equipement->GList_Port, i);
		if (Port_Equipement->Etat_Port == FIREWALLED) {
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Toujours pas de reponse  port %d", Port_Equipement->Num_Port);
			return FALSE;
		}
	}
	return TRUE;
}



void Scanne_Local_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port = NULL;
	int       Socket, i, Nb_Objet_Liste;

	ERROR_if (Equipement == NULL);
	Initialisation_Liste_Port_Equipement (Equipement);
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port = g_list_nth_data (Equipement->GList_Port, i);
		if ((Socket = ConnectionServeur ("127.0.0.1",NULL, Port->Num_Port, 1)) != -1) {
		
			Port->Etat_Port = OUVERT;
			if (close_control (Socket) < 0)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
		}
		else{
		
			Port->Etat_Port = FERMER;
		}
	}

}

void Scanne_Toutes_Les_Ports (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port = NULL;
	struct sockaddr sock;
	int       Socket, i, Tentative, Nb_Objet_Liste;

	ERROR_if (Equipement == NULL);
	if (memcmp (Equipement->ID_Equipement, Mon_Adresse_MAC, MAC_ADDR_LEN) == 0) {
		Scanne_Local_Equipement (Equipement);
		return;
	}

	Initialisation_Liste_Port_Equipement (Equipement);	
   
	strcpy (sock.sa_data, Info_Demarrage.Interface);
	sock.sa_family = AF_INET;
	if ((Socket = socket_control (AF_INET, SOCK_PACKET, htons (ETH_P_ALL))) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	Tentative = 0;	
	Nb_Objet_Liste = g_list_length (Equipement->GList_Port);
	do {
		for (i = 0; i < Nb_Objet_Liste; i++) {
			Port = g_list_nth_data (Equipement->GList_Port, i);
			if (Port->Etat_Port == FIREWALLED)
				Envoie_Trame_Scan_Port (Equipement, Port->Num_Port, Socket, &sock);
		}
		if ((!Info_Demarrage.Normal_Mode) && (Tentative == 2))
			break;
		
		switch (Tentative) {
		    case 0:
			    sleep (1);
			    break;
		    case 1:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Tentative 2");
			    sleep (1);
			    break;
		    case 2:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Tentative 3");
			    sleep (2);
			    break;
		    case 3:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Tentative 4");
			    sleep (3);
			    break;
		    case 4:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Tentative 5");
			    sleep (4);
			    break;
		    case 5:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Attend reponse");
			    sleep (4);
			    break;
		}
		Tentative++;
	}
	while (!(Test_Retour_Port_Equipement (Equipement)) && (Tentative != 6));
	debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Scanne port fini");
	if (close_control (Socket) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
}


void Creer_Liste_Port_Equipement (struct Struct_Liste_Equipement *Equipement)
{
	struct Struct_Port *Port_Equipement = NULL;
	struct struct_Liste_Port *Port = NULL;
	gint      Nb_Objet_Liste, i;

	ERROR_if (Equipement == NULL);
	Nb_Objet_Liste = g_list_length (GList_Port);
	for (i = 0; i < Nb_Objet_Liste; i++) {
		Port = g_list_nth_data (GList_Port, i);
		debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Port %d", Port->Num_Port);
		Port_Equipement = (struct Struct_Port *) malloc (sizeof (struct Struct_Port));
		ERROR_if (Port_Equipement == NULL);
		bzero (Port_Equipement, sizeof (struct Struct_Port));
		Port_Equipement->Num_Port = Port->Num_Port;
		Port_Equipement->Notification = FALSE;
		Port_Equipement->Niveau_Notification = NOTIFICATION_STANDARD;
		Port_Equipement->Etat_Port = FIREWALLED;
		Port_Equipement->Alerte_Port_Active = FALSE;
		Port_Equipement->Derniere_Modification_Etat_Port = 0;
		Port_Equipement->Compteur_Notification = 0;
		Equipement->GList_Port = g_list_append (Equipement->GList_Port, Port_Equipement);
}}

void Ajout_Un_Port (gpointer * Donnee)
{
	struct struct_Liste_Port *Port, *Donnee_Port = (struct struct_Liste_Port *) Donnee;

	ERROR_if (Donnee == NULL);
	Port = (struct struct_Liste_Port *) malloc (sizeof (struct struct_Liste_Port));
	ERROR_if (Port == NULL);
	bzero (Port, sizeof (struct struct_Liste_Port));
	Port->Num_Port = Donnee_Port->Num_Port;
	Port->Serveur = Donnee_Port->Serveur;
	debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Ajout port : %d", Port->Num_Port);
	GList_Port = g_list_append (GList_Port, Port);
}


void Initialisation_Liste_port (void)
{
	GList_Port = NULL;
}
