/* ***********************IMPORTANT *********************************************
 *  Copyright (C) 2004-2007 Thierry Lagarde <tlagarde@users.sourceforge.net>
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License.
 *
 *
 *                      ********************
 *
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU Library General Public License for more details.
 *
 *
 *                      ********************
 *
 *
 * You are highly encouraged to send your changes to <tlagarde@users.sourceforge.net>
 * for possible incorporation into the main distribution.
 * By sending these changes to <tlagarde@users.sourceforge.net>,
 * it is assumed that you are offering <tlagarde@users.sourceforge.net>
 * the unlimited, non-exclusive right to reuse, modify, and relicense the code.
 *
 *
 *                      ********************
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 **************************************************************************************/
#include <errno.h>
#include "Scan_Service.h"
#include "Debug.h"
#include "Reseau.h"
#include "Info.h"
#include <stdlib.h>

void Recupere_Nom_DNS (gchar * Adresse_IP_Char, gchar * DNS_Complet)
{
	struct hostent hostbuf, *host;
	size_t    Taille_Buffer;
	char     *Buffer, *Point, Temp_Chaine[TAILLE_CHAINE_NOM_DNS];
	int       erreur, Taille_Chaine;
	int       herr;
	long      addr;

	addr = inet_addr ((char *) Adresse_IP_Char);

	Taille_Buffer = 1024;

	Buffer = malloc (Taille_Buffer);

	while ((erreur = gethostbyaddr_r ((char *) &addr, sizeof (addr), AF_INET, &hostbuf, Buffer, Taille_Buffer, &host, &herr)) == ERANGE) {
	
		Taille_Buffer *= 2;
		Buffer = realloc (Buffer, Taille_Buffer);
	}

	if (herr || host == NULL) {
		switch (herr) {
		    case HOST_NOT_FOUND:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "L'hôte indiqué est inconnu.(%s) [%s]", hstrerror (herr), Adresse_IP_Char);
			    break;
		    case NO_ADDRESS:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Le nom est valide mais ne possède pas d'adresse IP.(%s) [%s]", hstrerror (herr), Adresse_IP_Char);
			    break;
		    case NO_RECOVERY:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Une erreur fatale du serveur de noms est apparue.(%s) [%s]", hstrerror (herr), Adresse_IP_Char);
			    break;
		    case TRY_AGAIN:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Aucune connection à  internet.(%s) [%s]", hstrerror (herr), Adresse_IP_Char);
			    break;
		    default:
			    debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur Inconnue(%d) pour [%s]", herr, Adresse_IP_Char);
			    break;
		}
	}
	else {
	
		Taille_Chaine = strlen (host->h_name);
		if (Taille_Chaine != 0) {
			if (Taille_Chaine > TAILLE_CHAINE_NOM_DNS)
				Taille_Chaine = TAILLE_CHAINE_NOM_DNS;
			strncpy (DNS_Complet, host->h_name, Taille_Chaine);
			DNS_Complet[Taille_Chaine] = 0;
		
		}
	}
	free (Buffer);	
}


char     *Recupere_Banniere_FTP (struct Struct_Liste_Equipement *Equipement)
{
#define   BANNIERE_FTP  "FTP : "
	int       IdSocket, NbOctet, TailleBanniere, TailleChaine;
	char      Requete[2000], *chaine = NULL, *banniere = NULL;
	FILE     *In = NULL;
	gboolean  fin;

	usleep (50000);	
	ERROR_if (Equipement == NULL);
	if ((IdSocket = ConnectionServeur (Equipement->Adresse_Ip_Char,NULL, PORT_FTP, Time_Out_Connect_Service)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur FTP [%s]", Equipement->Adresse_Ip_Char);
		return g_strdup ("Error FTP: Connection refused");
	}
	In = fdopen (IdSocket, "r");
	if (In == NULL) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		if (close_control (IdSocket) < 0)
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
		return g_strdup ("Error FTP :  In == NULL");
	}
	strcpy (Requete, "Bad command\n");
	NbOctet = send (IdSocket, Requete, strlen (Requete), 0);
	if (NbOctet < 0) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		fclose (In);
		if (close_control (IdSocket) < 0)
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
		return g_strdup ("Error FTP :  NbOctet < 0");
	}
	Nouveau_Octet_Envoye (NbOctet);
	TailleBanniere = sizeof (BANNIERE_FTP);
	TailleChaine = 0;
	banniere = malloc (sizeof (char) * TailleBanniere);
	ERROR_if (banniere == NULL);
	strcpy (banniere, BANNIERE_FTP);
	fin = FALSE;
	clearerr (In);	
	while ((!fin) && (attendre_Donnee (IdSocket, Time_Out_Service))) {
		chaine = fgets (Requete, sizeof (Requete), In);
		if (chaine == NULL) {
			if (close_control (IdSocket) < 0)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
			return g_strdup_printf ("Error FTP :  chaine == NULL  ferror(f)=%d", ferror (In));
		}
		while ((!fin) && (chaine != NULL)) {
			TailleChaine = strlen (chaine);
			Nouveau_Octet_Recu (TailleChaine);
			banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + TailleChaine)));
			strcat (banniere, chaine);
			TailleBanniere = TailleBanniere + TailleChaine;
			if (strncmp (chaine, "220", 3) != 0) {
				debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin des donnees a recuperer (FTP)");
				fin = TRUE;
			}
			chaine = fgets (Requete, sizeof (Requete), In);
		}
	}

	if (TailleBanniere != (strlen (banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere");




	if (close_Shutdown_control (In) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	fclose (In);
	return banniere;
}




char     *Recupere_Banniere_HTTP (struct Struct_Liste_Equipement *Equipement)
{
#define   BANNIERE_HTTP  "HTTP : "
	int       IdSocket, NbOctet, TailleBanniere, TailleChaine;
	char      Requete[2000], *chaine = NULL, *banniere = NULL;
	FILE     *In = NULL;
	gboolean  fin;

	usleep (50000);	
	ERROR_if (Equipement == NULL);
	if ((IdSocket = ConnectionServeur (Equipement->Adresse_Ip_Char, NULL, PORT_HTTP, Time_Out_Connect_Service)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur HTTP [%S]", Equipement->Adresse_Ip_Char);
		return g_strdup ("Error HTTP: Connection refused");
	}

	In = fdopen (IdSocket, "r");
	if (In == NULL) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		if (close_control (IdSocket) < 0)
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
		return g_strdup ("Error HTTP :  In == NULL");
	}

	sprintf (Requete, "GET / HTTP/1.1\r\nHost: %s\r\n\r\n", Equipement->Adresse_Ip_Char);

	NbOctet = send (IdSocket, Requete, strlen (Requete), 0);
	if (NbOctet < 0) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		fclose (In);
		if (close_control (IdSocket) < 0)
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
		return g_strdup ("Error HTTP :  NbOctet < 0");;
	}
	Nouveau_Octet_Envoye (NbOctet);
	TailleBanniere = sizeof (BANNIERE_HTTP);
	TailleChaine = 0;
	banniere = malloc (sizeof (char) * TailleBanniere);
	ERROR_if (banniere == NULL);
	strcpy (banniere, BANNIERE_HTTP);
	fin = FALSE;
	clearerr (In);	
	while ((!fin) && (attendre_Donnee (IdSocket, Time_Out_Service))) {
		chaine = fgets (Requete, sizeof (Requete), In);
		if (chaine == NULL) {
			if (close_control (IdSocket) < 0)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
			return g_strdup_printf ("Error HTTP :  chaine == NULL  ferror(f)=%d", ferror (In));
		}
		while ((!fin) && (chaine != NULL)) {
			TailleChaine = strlen (chaine);
			Nouveau_Octet_Recu (TailleChaine);
			if (strncmp (chaine, "\r\n", 2) != 0) {
				banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + TailleChaine)));
				strcat (banniere, chaine);
				TailleBanniere = TailleBanniere + TailleChaine;
			}
			else {
				debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin des donnees a recuperer (HTTP)");
				fin = TRUE;
			}
			chaine = fgets (Requete, sizeof (Requete), In);
		}
	}

	if (TailleBanniere != (strlen (banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere");

	if (close_Shutdown_control (In) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	fclose (In);
	return banniere;
}


char     *Recupere_Banniere_IMAP (struct Struct_Liste_Equipement *Equipement)
{
#define   BANNIERE_IMAP  "IMAP : "
	int       IdSocket, TailleBanniere, TailleChaine;
	char      Requete[2000], *chaine = NULL, *banniere = NULL;
	FILE     *In = NULL;
	gboolean  fin;

	usleep (50000);	
	ERROR_if (Equipement == NULL);
	if ((IdSocket = ConnectionServeur (Equipement->Adresse_Ip_Char,NULL, PORT_IMAP, Time_Out_Connect_Service)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur IMAP [%s]", Equipement->Adresse_Ip_Char);
		return g_strdup ("Error IMAP: Connection refused");
	}
	In = fdopen (IdSocket, "r");
	if (In == NULL) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
		if (close_control (IdSocket) < 0)
			debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
		return g_strdup ("Error IMAP :  In == NULL");
	}
	TailleBanniere = sizeof (BANNIERE_IMAP);
	TailleChaine = 0;
	banniere = malloc (sizeof (char) * TailleBanniere);
	ERROR_if (banniere == NULL);
	strcpy (banniere, BANNIERE_IMAP);
	fin = FALSE;
	clearerr (In);	
	while ((!fin) && (attendre_Donnee (IdSocket, Time_Out_Service))) {
		chaine = fgets (Requete, sizeof (Requete), In);
		if (chaine == NULL) {
			if (close_control (IdSocket) < 0)
				debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
			return g_strdup_printf ("Error IMAP :  chaine == NULL  ferror(f)=%d", ferror (In));
		}
		while ((!fin) && (chaine != NULL)) {
			Nouveau_Octet_Recu (strlen (chaine));
			TailleChaine = strlen (chaine);
			banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + TailleChaine)));
			strcat (banniere, chaine);
			TailleBanniere = TailleBanniere + TailleChaine;
			if (strstr (chaine, "\r\n") == NULL) {
				debug_printf (D_ALL, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Fin des donnees a recuperer (IMAP)");
				fin = TRUE;
			}
			chaine = fgets (Requete, sizeof (Requete), In);
		}
	}

	if (TailleBanniere != (strlen (banniere) + 1))
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Erreur TailleBanniere");





	if (close_Shutdown_control (In) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	fclose (In);
	return banniere;
}


char     *Recupere_Banniere_TELNET (struct Struct_Liste_Equipement *Equipement)
{
#define   BANNIERE_TELNET  "TELNET : "
	int       IdSocket, NbOctet, Telnet_Recuperer, i, TailleBanniere, j;
	char      DonneeBrut[2000], *banniere = NULL;

	usleep (50000);	
	ERROR_if (Equipement == NULL);
	Telnet_Recuperer = -1;
	if ((IdSocket = ConnectionServeur (Equipement->Adresse_Ip_Char,NULL,PORT_TELNET, Time_Out_Connect_Service)) == -1) {
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Connexion refusée au serveur TELNET [%s]", Equipement->Adresse_Ip_Char);
		return g_strdup ("Error TELNET: Connection refused");
	}
	TailleBanniere = sizeof (BANNIERE_TELNET);
	banniere = malloc (sizeof (char) * TailleBanniere);
	ERROR_if (banniere == NULL);
	strcpy (banniere, BANNIERE_TELNET);
	while ((Telnet_Recuperer == -1) && ((attendre_Donnee (IdSocket, Time_Out_Service) != FALSE) && ((NbOctet = recvfrom (IdSocket, &DonneeBrut, (sizeof (DonneeBrut) - 1), 0, NULL, 0)) > 0))) {
		Nouveau_Octet_Recu (NbOctet);
		if ((DonneeBrut[0] & 0xff) == 0xff) {
			i = 0;
			while ((i < (NbOctet / 3)) && ((DonneeBrut[(i * 3)] & 0xff) == 0xff)) {
				DonneeBrut[(i * 3) + 1] = 0xfc;
				i++;
			}
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%s)Taille retour negociation : [%d]  total:[%d]", Equipement->Adresse_Ip_Char, (i * 3), NbOctet);
			if ((i * 3) < NbOctet) {
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%s)Des donnees après la negociation  taille(%d)", Equipement->Adresse_Ip_Char, (NbOctet - (i * 3)));
				/*for (j = 0; j < NbOctet; j++) {
				 * printf ("[%d]", DonneeBrut[j]);
				 * }
				 * printf ("\n"); */
			
			
			
			
			
			
			
				for (j = (i * 3); j < NbOctet; j++)
					if ((DonneeBrut[j] != 10) && (DonneeBrut[j] != 13) && ((DonneeBrut[j] < 31) || (DonneeBrut[j] > 126)))
						DonneeBrut[j] = '$';
				DonneeBrut[NbOctet] = 0;
				TailleBanniere = TailleBanniere + (NbOctet - (i * 3));
				banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + 1)));
				strcat (banniere, DonneeBrut + (i * 3));
				debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%s) %s [%d]  TailleBanniere[%d] ", Equipement->Adresse_Ip_Char, banniere, strlen (banniere), TailleBanniere);
			}
			NbOctet = send (IdSocket, DonneeBrut, (i * 3), 0);
			if (NbOctet < 0) {
				debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s", strerror (errno));
				if (close_control (IdSocket) < 0)
					debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
				return g_strdup ("Error TELNET: NbOctet < 0");
			}
			Nouveau_Octet_Envoye (NbOctet);
		}
		else {
			for (j = 0; j < NbOctet; j++)
				if ((DonneeBrut[j] != 10) && (DonneeBrut[j] != 13) && ((DonneeBrut[j] < 31) || (DonneeBrut[j] > 126)))
					DonneeBrut[j] = '$';
			DonneeBrut[NbOctet] = 0;
			TailleBanniere = TailleBanniere + NbOctet;
			banniere = realloc (banniere, (sizeof (char) * (TailleBanniere + 1)));
			strcat (banniere, DonneeBrut);
			debug_printf (D_INFO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "Standard - (%s) %s [%d] TailleBanniere[%d] ", Equipement->Adresse_Ip_Char, banniere, strlen (banniere), TailleBanniere);
		}
	}

	if (TailleBanniere != (strlen (banniere) + 1))
		debug_printf (D_WARN, __FILE__, __PRETTY_FUNCTION__, __LINE__, "(%s) TailleBanniere : (%d)!=(%d)", Equipement->Adresse_Ip_Char, TailleBanniere, (strlen (banniere) + 1));
	if (close_control (IdSocket) < 0)
		debug_printf (D_ERRO, __FILE__, __PRETTY_FUNCTION__, __LINE__, "%s\nSocket ouvert[%d]  Thread[%d][%d]\n Nb equipement (%d)", strerror (errno), Nb_Socket_Ouverte, Info_Demarrage.Nb_Thread_Max, Info_Demarrage.Nb_Thread_Max - Valeur_Semaphore (&Semaphore_Thread_Scan), g_list_length (GList_Equipement));
	return banniere;

}
